@@ -160,7 +160,14 @@ class TerseJsonTestCase(LoggerCleanupMixin, TestCase):
         logger = self.get_logger(handler)
 
         # A full request isn't needed here.
-        site = Mock(spec=["site_tag", "server_version_string", "getResourceFor"])
+        site = Mock(
+            spec=[
+                "site_tag",
+                "server_version_string",
+                "getResourceFor",
+                "_parsePOSTFormSubmission",
+            ]
+        )
         site.site_tag = "test-site"
         site.server_version_string = "Server v1"
         site.reactor = Mock()
