@@ -324,7 +324,7 @@ class EventsStreamTestCase(BaseStreamTestCase):
         pls = self.helper.get_state(
             self.room_id, EventTypes.PowerLevels, tok=self.user_tok
         )
-        pls["users"].update({u: 50 for u in user_ids})
+        pls["users"].update(dict.fromkeys(user_ids, 50))
         self.helper.send_state(
             self.room_id,
             EventTypes.PowerLevels,
